#include "setiview.h"
#include <string.h>
#include <stdio.h>
#include <time.h>

extern WINDOW *machine;

static char hostname_string[128] = "";
static char datetime_string[128] = "";
static char loadavg_string[128] = "";

void machine_set_hostname(const char *name)
{
	strncpy(hostname_string, name, 128);
}

static void draw_machine_hostname(void)
{
	wmove(machine, 1, 0);
	waddstring(machine, hostname_string, A_NORMAL);
}

static void draw_machine_datetime(void)
{
	time_t t;

	t = time(NULL);
	strncpy(datetime_string, ctime(&t), 128);
	wmove(machine, 2, 0);
	waddstring(machine, datetime_string, A_NORMAL);
}

static void draw_machine_loadavg(void)
{
	FILE *uptime;
	char buf[128];

/* linux-specific (and better optimized) load average routine */
#ifdef linux
	char *space;
	uptime = fopen("/proc/loadavg", "r");
	if(!uptime) return;
	fgets(buf, 128, uptime);
	fclose(uptime);
	/* find third space */
	space = strchr(buf, ' ');
	space = strchr(space+1, ' ');
	space = strchr(space+1, ' ');
	/* mark it the end of the string */
	*space = 0;
	sprintf(loadavg_string, "load average: %s", buf);
#else
	char *load;
	static time_t last_update_time = 0;
	time_t current_time;

	current_time = time(NULL);
	if(current_time - last_update_time < 60) return;
	last_update_time = current_time;

	uptime = popen("uptime", "r");
	if(!uptime) return;
	fgets(buf, 128, uptime);
	pclose(uptime);
	load = strstr(buf, "load");
	strcpy(loadavg_string, load);
	loadavg_string[strlen(loadavg_string) - 1] = '\0';	/* strip \n */
#endif

	wmove(machine, 3, 0);
	waddstring(machine, loadavg_string, A_NORMAL);
}

void machine_redraw(void)
{
	const char *machine_title = "Host Info";
	wmove(machine, 0, (COLS/2 - strlen(machine_title))/2);
	waddstring(machine, machine_title, A_BOLD);

	draw_machine_hostname();
	draw_machine_datetime();
	draw_machine_loadavg();

	wnoutrefresh(machine);
}

