#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include "setiview.h"
#include "pty.h"

extern void yylex(void);
extern FILE *yyin;

static int seti_pid=-1;
static char **seti_argv;
static char verbose[]="-verbose";
int interrupted=0;
static int curses_initialized=0, pty_open=0;

void int_handler(int sig)
{
	interrupted = 1;
	if(seti_pid!=-1) kill(SIGHUP, seti_pid);
}

void winch_handler(int sig)
{
	setiview_end();
	setiview_init();
}

void child_handler(int sig)
{
	int status, pid;
	pid = wait(&status);
	if(pid != seti_pid) return;
	if(!interrupted && curses_initialized) {
	    if(WIFEXITED(status))
		analysis_status("setiathome exited!");
	    if(WIFSIGNALED(status))
		analysis_status("setiathome killed!");
	    else return; /* don't worry about stopped processes */
	}
	seti_pid=-1;
	if(pty_open) pty_close();
	pty_open = 0;
}

void seti_restart(void)
{
	if(interrupted) return;
	analysis_status("restarting setiathome...");
/* setiathome should already be dead */
	if(seti_pid != -1)
		kill(seti_pid, SIGTERM);
	sleep(5);
	if(pty_open) pty_close();
	yyin = pty_alloc();
	pty_open = 1;
	pty_fork(seti_argv, &seti_pid);
}

int main(int argc, char **argv)
{
	bool verbose_found = FALSE;
	int i;
	
	if(argc < 2) {
		printf("usage:\n%s [setiathome commandline]\n", argv[0]);
		return 0;
	}

	seti_argv = (char **) calloc( (size_t) argc + 1, (size_t) sizeof( char *));
	
	// "-verbose" entered at cmdline ? we need this for displaying something useful in "data analysis"

	for( i=0; i < argc-1; i++ )
	{
		*(seti_argv+i) = *(argv+i+1);
		verbose_found = verbose_found || ( strcmp( *(seti_argv+i), verbose ) == 0);
	} 

	// no, add it.

	if( !verbose_found )
	{
		*(seti_argv+i) = verbose;
		*(seti_argv+i+1) = NULL;
	};

	yyin = pty_alloc();
	pty_open = 1;
	pty_fork(seti_argv, &seti_pid);

	signal(SIGWINCH, winch_handler);
	signal(SIGCHLD, child_handler);
	signal(SIGINT, int_handler);

	setiview_init();
	curses_initialized=1;
	yylex();
	curses_initialized=0;
	setiview_end();

	if(seti_pid != -1) kill(SIGHUP, seti_pid);
	if(pty_open) pty_close();

	free( seti_argv );

	return 0;
}

