#include "setiview.h"
#include "conf.h"
#include <stdio.h>
#include <unistd.h>

WINDOW *analysis, *datainfo, *userinfo, 
	*progress, *machine, *misc, *shameless_plug;

static int seti_major=-1, seti_minor=-1;
void get_seti_version(void)
{
	conf_struct *cs = conf_open("version.sah");
	if(!cs) return;
	conf_get_int(cs, "major_version", &seti_major);
	conf_get_int(cs, "minor_version", &seti_minor);
	conf_close(cs);
}

/* curses should really have something like this built in. */
void waddstring(WINDOW *win, const char *string, chtype attr)
{
	for(;*string;string++) waddch(win, (*string)|attr);
}

void setiview_refresh(WINDOW *win)
{
	if(win!=progress) wnoutrefresh(win);
	wnoutrefresh(progress);
	doupdate();
}

void setiview_init(void)
{
	char machine_hostname[40];
	gethostname(machine_hostname, 40);
	machine_set_hostname(machine_hostname);

	get_seti_version();

	initscr();
	noecho();
	cbreak();
	nonl();

	shameless_plug = newwin(1, 0, 0, 0);
	userinfo = newwin(6, COLS/2-1, 1, 0);
	datainfo = newwin(6, COLS/2, 1, COLS/2);
	analysis = newwin(6, COLS/2-1, 7, 0);
	machine = newwin(6, COLS/2, 7, COLS/2);
	progress = newwin(1, 0, 13, 0);
	misc = newwin(LINES-14, 0, 14, 0);

	scrollok(shameless_plug, FALSE);
	scrollok(userinfo, FALSE);
	scrollok(datainfo, FALSE);
	scrollok(analysis, FALSE);
	scrollok(machine, FALSE);
	scrollok(progress, FALSE);
	scrollok(misc, TRUE);

	wbkgdset(shameless_plug, A_REVERSE|' ');

	user_info_init();

	setiview_redraw();
}

void setiview_end(void)
{
	delwin(userinfo);
	delwin(datainfo);
	delwin(analysis);
	delwin(progress);
	delwin(machine);
	delwin(misc);
	endwin();
}

void setiview_redraw(void)
{
	char buf[128];
	char *plug = "SETIView "VERSION;
	if(seti_major!=-1)
	    sprintf(buf, "%s: SETI@home %d.%d", plug, seti_major, seti_minor);
	else
	    strcpy(buf, plug);
	werase(shameless_plug);
	wmove(shameless_plug, 0, (COLS - strlen(buf))/2);
	waddstring(shameless_plug, buf, A_REVERSE);
	wnoutrefresh(shameless_plug);
	
	machine_redraw();
	user_info_redraw();
	data_info_redraw();
	analysis_redraw();
	progress_redraw();
	doupdate();
}

void misc_put(char c)
{
	static int col=0;
#ifdef MISC_DEBUG /* this can safely be removed, i guess... */
	static FILE *fptmp = NULL;
	if(!fptmp) fptmp = fopen("misc.txt", "w");
#endif
	if(!c) return;
	if(c=='\r') return;
	if(c=='\n') {
		if(col == 0) return;
		waddch(misc, c|A_NORMAL);
#ifdef MISC_DEBUG
		fputc(c, fptmp);
#endif
		misc_flush();
		col = 0;
		return;
	}
	if(c==' ' && col == 0) return;
	waddch(misc, c|A_NORMAL);
#ifdef MISC_DEBUG
	fputc(c, fptmp);
#endif
	col++;
}

void misc_flush(void)
{
	setiview_refresh(misc);
}

