#include <string.h>
#include "setiview.h"
#include "conf.h"

extern WINDOW *userinfo;

static char cputime_string[128] = "";
static char workunits_string[128] = "";
static char login_string[128] = "";
static char sessionwork_string[128] = "Work units this session: 0";
static int work_units_this_session = 0;

static void draw_userinfo(int line, const char *string)
{
	wmove(userinfo, line, 0);
	wclrtoeol(userinfo);
	waddstring(userinfo, string, A_NORMAL);
}

void user_info_init(void)
{
	char email[128], name[128];
	float cputime;
	int hr, min, sec, seconds, units;
	int have_email, have_name;
	conf_struct *cs;
	cs = conf_open("user_info.sah");
	if(!cs) return;
	have_email = conf_get_string(cs, "email_addr", email);
	have_name = conf_get_string(cs, "name", name);
	if(have_email && have_name) 
		sprintf(login_string, "%.32s (%.88s)", name, email);
	else if(have_email)
		strncpy(login_string, email, 128);
	else if(have_name)
		strncpy(login_string, name, 128);
	
	if(conf_get_float(cs, "total_cpu", &cputime)) {
		seconds = cputime;
		hr = seconds/3600; seconds = seconds%3600;
		min = seconds/60; sec = seconds%60;
		sprintf(cputime_string, "Total CPU time: %d hr %d min %d sec",
			hr, min, sec);	
	}
	if(conf_get_int(cs, "nresults", &units)) {
		sprintf(workunits_string, "Work units processed: %d", units);
	}

	conf_close(cs);
}

void user_info_redraw(void)
{
	const char *userinfo_title = "User Info";
	wmove(userinfo, 0, (COLS/2 - strlen(userinfo_title))/2);
	waddstring(userinfo, userinfo_title, A_BOLD);

	draw_userinfo(1, login_string);
	draw_userinfo(2, cputime_string);
	draw_userinfo(3, workunits_string);
	draw_userinfo(4, sessionwork_string);

	wnoutrefresh(userinfo);
}

void user_info_login(const char *name)
{
	strncpy(login_string, name, 128);
	draw_userinfo(1, login_string);
	setiview_refresh(userinfo);
}

void user_info_workunits(int units)
{
	sprintf(workunits_string, "Work units processed: %d", units);
	draw_userinfo(3, workunits_string);
	setiview_refresh(userinfo);
}

void user_info_cputime(int seconds)
{
	int hr, min, sec;
	hr = seconds/3600; seconds = seconds%3600;
	min = seconds/60; sec = seconds%60;
	sprintf(cputime_string, "Total CPU time: %d hr %d min %d sec",
		hr, min, sec);	
	draw_userinfo(2, cputime_string);
	setiview_refresh(userinfo);
}

void user_info_workunit_complete(void)
{
	work_units_this_session++;
	sprintf(sessionwork_string, "Work units this session: %d", 
		work_units_this_session);
	draw_userinfo(4, sessionwork_string);
	setiview_refresh(userinfo);
}

